/*##########################################################################
 #   Amarok script for interfacing with SoundCloud                         #
 #                                                                         #
 #   Copyright                                                             #
 #   (C) 2014 Be <be.0@gmx.com>                                            #
 #                                                                         #
 #   This program is free software; you can redistribute it and/or modify  #
 #   it under the terms of the GNU General Public License as published by  #
 #   the Free Software Foundation; either version 3 of the License, or     #
 #   (at your option) any later version.                                   #
 #                                                                         #
 #   Note that if you modify this software and continue to use the         #
 #   SoundCloud API in your modified version, the SoundCloud API Terms of  #
 #   Use (https://developers.soundcloud.com/docs/api/terms-of-use ) state, #
 #   in the section General Prohibitions, clause xiv:                      #
 #      You must not use or attempt to use another person's client ID      #
 #      and/or client secret, unless you are working on the same app.      #
 #   You are encouraged to communicate your modifications to the author    #
 #   so they can be incorporated into a future version of this program.    #
 #   If you wish to publish a modified version, to comply with the         #
 #   SoundCloud API Terms of Use, register your new program with           #
 #   SoundCloud through their website and replace the value of the global  #
 #   variable client_id, defined at the first line of code, with your      #
 #   client ID.                                                            #
 #                                                                         #
 #   This program is distributed in the hope that it will be useful,       #
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          #
 #   GNU General Public License for more details.                          #
 #                                                                         #
 #   You should have received a copy of the GNU General Public License     #
 #   along with this program; if not, write to the                         #
 #   Free Software Foundation, Inc.,                                       #
 #   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         #
 #########################################################################*/

function groupList(reply) {
	query = JSON.parse(reply);
	
	if (! ('errors' in query)) {
		item = Amarok.StreamItem;
		item.level = 2;
		item.coverUrl = '';
		item.playableUrl = '';
		item.year = '';
		for (i in query) {
			item.itemName = query[i].name;
			item.artist = query[i].creator.username;
			item.playableUrl = 'group ' + query[i].name;
			item.infoHtml = htmlify(query[i].description);
			item.coverUrl = query[i].artwork_url;
			if (query[i].track_count > 0) {
				item.callbackData = JSON.stringify(query[i]);
			} else {
				item.callbackData = '{"errors": "no tracks"}';
			}
			script.insertItem(item);
			Amarok.debug(item.itemName + ' inserted');
		}
	}
	script.donePopulating();
	Amarok.debug('Group list populated');
}

function groupTracks (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.itemName = 'Tracks';
	item.callbackData = reply;
	script.insertItem(item);
	new Downloader(new QUrl('https://api.soundcloud.com/groups/' + id + '/members?client_id=' + client_id), groupMembers);
}

function groupMembers (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.itemName = 'Members';
	item.callbackData = reply;
	script.insertItem(item);
	new Downloader(new QUrl('https://api.soundcloud.com/groups/' + id + '/contributors?client_id=' + client_id), groupContributors);
}

function groupContributors (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.itemName = 'Contributors';
	item.callbackData = reply;
	script.insertItem(item);
	new Downloader(new QUrl('https://api.soundcloud.com/groups/' + id + '/moderators?client_id=' + client_id), groupModerators);
}

function groupModerators (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.itemName = 'Moderators';
	item.callbackData = reply;
	script.insertItem(item);
	script.donePopulating();
}