/*##########################################################################
 #   Amarok script for interfacing with SoundCloud                         #
 #                                                                         #
 #   Copyright                                                             #
 #   (C) 2014 Be <be.0@gmx.com>                                            #
 #                                                                         #
 #   This program is free software; you can redistribute it and/or modify  #
 #   it under the terms of the GNU General Public License as published by  #
 #   the Free Software Foundation; either version 3 of the License, or     #
 #   (at your option) any later version.                                   #
 #                                                                         #
 #   Note that if you modify this software and continue to use the         #
 #   SoundCloud API in your modified version, the SoundCloud API Terms of  #
 #   Use (https://developers.soundcloud.com/docs/api/terms-of-use ) state, #
 #   in the section General Prohibitions, clause xiv:                      #
 #      You must not use or attempt to use another person's client ID      #
 #      and/or client secret, unless you are working on the same app.      #
 #   You are encouraged to communicate your modifications to the author    #
 #   so they can be incorporated into a future version of this program.    #
 #   If you wish to publish a modified version, to comply with the         #
 #   SoundCloud API Terms of Use, register your new program with           #
 #   SoundCloud through their website and replace the value of the global  #
 #   variable client_id, defined at the first line of code, with your      #
 #   client ID.                                                            #
 #                                                                         #
 #   This program is distributed in the hope that it will be useful,       #
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          #
 #   GNU General Public License for more details.                          #
 #                                                                         #
 #   You should have received a copy of the GNU General Public License     #
 #   along with this program; if not, write to the                         #
 #   Free Software Foundation, Inc.,                                       #
 #   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         #
 #########################################################################*/

function userList(reply) {
	query = JSON.parse(reply);
	
	if (! ('errors' in query)) {
		item = Amarok.StreamItem;
		item.level = 2;
		for (i in query) {
			item.itemName = query[i].username;
			item.artist = query[i].username;
			item.coverUrl = query[i].avatar_url;
			item.playableUrl = '';
			item.infoHtml = htmlify(query[i].description);
			if (query[i].track_count > 0) {
				item.callbackData = JSON.stringify(query[i]);
			} else {
				item.callbackData = '{"errors": "no tracks"}';
			}
			script.insertItem(item);
			Amarok.debug(item.itemName + ' inserted');
		}
	}
	script.donePopulating();
	Amarok.debug('Artist list populated');
}

function userPlaylists(reply) {
	plists = JSON.parse(reply);
	playlistTracks = [];
	if (! (plists.length == 0)) {
		item = Amarok.StreamItem;
		item.level = 1;
		item.playableUrl = '';
		for (i in plists) {
			item.callbackData = JSON.stringify(plists[i].tracks);
			item.year = plists[i].release_year;
			item.itemName = plists[i].title;
			item.artist = plists[i].user.username;
			item.infoHtml = htmlify(plists[i].description);
			item.coverUrl = plists[i].artwork_url;
			script.insertItem(item);
			for (j in plists[i].tracks) {
				playlistTracks.push(plists[i].tracks[j]);
			}
		}
	}
	new Downloader(new QUrl('https://api.soundcloud.com/users/' + id + '/tracks?client_id=' + client_id ), userNonPlaylistTracks);
	new Downloader(new QUrl('https://api.soundcloud.com/users/' + id + '/favorites?client_id=' + client_id ), userFavorites);
	new Downloader(new QUrl('https://api.soundcloud.com/users/' + id + '/followings?client_id=' + client_id ), userFollowing);
	new Downloader(new QUrl('https://api.soundcloud.com/users/' + id + '/followers?client_id=' + client_id ), userFollowers);
	new Downloader(new QUrl('https://api.soundcloud.com/users/' + id + '/groups?client_id=' + client_id ), userGroups);
}

function userNonPlaylistTracks (reply) {
	tracklist = JSON.parse(reply);
	cb = JSON.stringify(tracklist.filter(userNotInPlaylist));
	if (cb != '[]') {
		item = Amarok.StreamItem;
		item.level = 1;
		item.callbackData = cb;
		item.itemName = 'Other tracks';
		item.artist = tracklist[0].user.username;
		item.coverUrl = tracklist[0].user.avatar_url;
		script.insertItem(item);
	}
}

function userNotInPlaylist (value, index, array) {
	for (i in playlistTracks) {
		if (value.title == playlistTracks[i].title) {
			return false;
		}
	}
	return true;
}

function userFavorites (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.callbackData = reply;
	item.itemName = 'Favorites';
	script.insertItem(item);
}

function userFollowing (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.callbackData = reply;
	item.itemName = 'Following';
	script.insertItem(item);
}

function userFollowers (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.callbackData = reply;
	item.itemName = 'Followers';
	script.insertItem(item);
}

function userGroups (reply) {
	item = Amarok.StreamItem;
	item.level = 1;
	item.callbackData = reply;
	item.itemName = 'Groups';
	script.insertItem(item);
	script.donePopulating();
}